import { BaseClient, BaseBody } from '@dvcol/base-http-client';
import { RecursiveRecord } from '@dvcol/common-utils/common/models';
import { SimklApi } from '../api/simkl-api.endpoints.cjs';
import { SimklApiResponse, SimklApiQuery, SimklClientSettings, SimklClientAuthentication, SimklClientOptions, ISimklApi, SimklApiParams, SimklApiTemplate } from '../models/simkl-client.model.cjs';
import '../models/simkl-activity.model.cjs';
import '../models/simkl-anime.model.cjs';
import '../models/simkl-common.model.cjs';
import '../models/simkl-episode.model.cjs';
import '../models/simkl-rating.model.cjs';
import '../models/simkl-authentication.model.cjs';
import '../models/simkl-calendar.model.cjs';
import '../models/simkl-checkin.model.cjs';
import '../models/simkl-movie.model.cjs';
import '../models/simkl-show.model.cjs';
import '../models/simkl-redirect.model.cjs';
import '../models/simkl-search.model.cjs';
import '../models/simkl-sync.model.cjs';
import '../models/simkl-user.model.cjs';
import '@dvcol/common-utils/http/fetch';

/**
 * Checks if the fetch response is OK and handles redirects.
 *
 * @private
 *
 * @param  response - The fetch response.
 *
 * @returns  The same response object if OK.
 *
 * @throws Throws the response if not OK.
 */
declare const isResponseOk: (response: Response) => Response;
/**
 * Parses a API response to extract {@link SimklApiClientPagination} information.
 *
 * @private
 *
 * @template T - The type of the response.
 *
 * @param {Response} response - The fetch response.
 *
 * @returns {SimklApiResponse<T>} The parsed API response.
 */
declare const parseResponse: <T>(response: SimklApiResponse<T>) => SimklApiResponse<T>;
/** Needed to type Object assignment */
interface BaseSimklClient extends SimklApi {
}
/**
 * Represents a Mal API client with common functionality.
 *
 * @class BaseSimklClient
 */
declare class BaseSimklClient extends BaseClient<SimklApiQuery, SimklApiResponse, SimklClientSettings, SimklClientAuthentication> implements SimklApi {
    /**
     * Creates an instance of BaseMalClient.
     * @param options - The options for the client.
     * @param authentication - The authentication for the client.
     * @param api - The API endpoints for the client.
     */
    constructor(options: SimklClientOptions, authentication?: SimklClientAuthentication, api?: ISimklApi);
    /**
     * Parses the template to construct the headers for a Mal API request.
     *
     * @protected
     *
     * @template T - The type of the parameters.
     *
     * @param {SimklApiTemplate<T>} template - The template for the API endpoint.
     *
     * @returns {HeadersInit} The parsed request headers.
     *
     * @throws {Error} Throws an error if OAuth is required and the access token is missing.
     */
    protected _parseHeaders<T extends SimklApiParams = SimklApiParams>(template: SimklApiTemplate<T>): HeadersInit;
    /**
     * Parses the parameters and constructs the URL for a Mal API request.
     *
     * @protected
     *
     * @template T - The type of the parameters.
     *
     * @param template - The template for the API endpoint.
     * @param {T} params - The parameters for the API call.
     *
     * @returns {string} The URL for the Mal API request.
     *
     * @throws {Error} Throws an error if mandatory parameters are missing or if a filter is not supported.
     */
    protected _parseUrl<T extends SimklApiParams = SimklApiParams>(template: SimklApiTemplate<T>, params: T): URL;
    /**
     * Parses body from a template and stringifies a {@link BodyInit}
     *
     * @protected
     *
     * @template T - The type of the parameters.
     *
     * @param template - The expected body structure.
     * @param {T} params - The actual parameters.
     *
     * @returns {BodyInit} The parsed request body.
     */
    protected _parseBody<T extends SimklApiParams = SimklApiParams>(template: BaseBody<string | keyof T>, params: T): BodyInit;
    /**
     * Parses the response from the API before returning from the call.
     * @param response - The response from the API.
     *
     * @returns {SimklApiResponse} The parsed response.
     * @protected
     */
    protected _parseResponse<T extends RecursiveRecord = unknown>(response: SimklApiResponse<T>): SimklApiResponse;
}

export { BaseSimklClient, isResponseOk, parseResponse };
