import { BaseSimklClient } from './base-simkl-client.js';
import * as _dvcol_base_http_client from '@dvcol/base-http-client';
import { SimklAuthenticationCode, SimklAuthorizeQuery, SimklTokenExchangeQuery } from '../models/simkl-authentication.model.js';
import { SimklClientAuthentication, SimklClientOptions, ISimklApi } from '../models/simkl-client.model.js';
import '@dvcol/common-utils/common/models';
import '../api/simkl-api.endpoints.js';
import '../models/simkl-activity.model.js';
import '../models/simkl-anime.model.js';
import '../models/simkl-common.model.js';
import '../models/simkl-episode.model.js';
import '../models/simkl-rating.model.js';
import '@dvcol/common-utils/http/fetch';
import '../models/simkl-calendar.model.js';
import '../models/simkl-checkin.model.js';
import '../models/simkl-movie.model.js';
import '../models/simkl-show.model.js';
import '../models/simkl-redirect.model.js';
import '../models/simkl-search.model.js';
import '../models/simkl-sync.model.js';
import '../models/simkl-user.model.js';

type CancellablePolling = Promise<SimklClientAuthentication> & {
    cancel: () => void;
};
/**
 * MalClient is a wrapper around the MalApi to provide basic authentication and state management.
 *
 * @class SimklClient
 *
 * @extends {BaseSimklClient}
 */
declare class SimklClient extends BaseSimklClient {
    protected polling: ReturnType<typeof setTimeout> | undefined;
    protected poll: SimklAuthenticationCode | undefined;
    /**
     * The url to redirect to after the user has authorized the application.
     */
    get redirectUri(): string;
    /**
     * Creates an instance of MalClient, with the necessary endpoints and settings.
     * @param settings - The settings for the client.
     * @param authentication - The authentication for the client.
     * @param api - The API endpoints for the client.
     */
    constructor(settings: SimklClientOptions, authentication?: SimklClientAuthentication, api?: ISimklApi);
    protected _clearPolling(): void;
    protected _clearPoll(): void;
    /**
     * Initiates the OAuth process by redirecting to the Simkl website.
     * Users will be prompted to sign in and authorize the application.
     *
     * Once redirected back to the application, the code should be exchanged for an access token.
     *
     * @param redirect - The type of redirect to use (defaults to manual).
     * @param redirect_uri - The URL to redirect to after the user has authorized the application (defaults to client settings).
     * @param request.code_challenge - The code challenge to match against the code verifier.
     * @param request.state - The optional CSRF token to verify the state.
     * @param request - Additional parameters for the authorization request.
     * @returns A promise resolving to the response from the Simkl website.
     *
     * @see [authorize]{@link https://simkl.docs.apiary.io/#reference/authentication-oauth-2.0/authorize/authorize-application}
     */
    authorize({ redirect, redirect_uri, ...request }?: SimklAuthorizeQuery): Promise<_dvcol_base_http_client.TypedResponse<unknown>>;
    /**
     * Initiates the OAuth process by generating a URL to the Simkl website.
     * Users will be prompted to sign in and authorize the application.
     *
     * Once redirected back to the application, the code should be exchanged for an access token with {@link exchangeCodeForToken}.
     *
     * @param redirect_uri - The URL to redirect to after the user has authorized the application (defaults to client settings).
     * @param request - Additional parameters for the authorization request.
     * @returns A promise resolving to the response from the Simkl website.
     *
     * @see [authorize]{@link https://simkl.docs.apiary.io/#reference/authentication-oauth-2.0/authorize/authorize-application}
     */
    resolveAuthorizeUrl({ redirect_uri, ...request }?: Omit<SimklAuthorizeQuery, 'redirect'>): string;
    /**
     * Exchanges the authorization code obtained after the user has authorized the application with {@link authorize} or {@link resolveAuthorizeUrl}.
     *
     * @param  code - The authorization code obtained from the user.
     * @param code_verifier - The code verifier to match again the code challenge.
     * @param redirect_uri - The URL to redirect to after the user has authorized the application (defaults to client settings).
     * @param  state - The optional CSRF token to verify the state.
     *
     * @returns  A promise resolving to the Trakt authentication information.
     *
     * @throws Error Throws an error if the CSRF token is invalid.
     */
    exchangeCodeForToken({ code, redirect_uri }: SimklTokenExchangeQuery, state?: string): Promise<SimklClientAuthentication>;
    /**
     * Gets the device code for initiating device authentication.
     *
     * The code should then be used in conjunction with the {@link pollWithDeviceCode} method to finish authentication.
     *
     * @returns A promise resolving to the device authentication information.
     */
    getDeviceCode(redirect?: string): Promise<SimklAuthenticationCode>;
    /**
     * Polls the device authentication endpoint to complete the authentication.
     * If the timeout is reached, the polling is cancelled and an error is thrown.
     * If the authentication is successful, the polling is cancelled and the authentication information is returned.
     *
     * @param poll - The device authentication information.
     * @param timeout - The timeout in milliseconds.
     *
     * @returns A promise resolving to the authentication information if successful
     */
    protected _devicePolling(poll: SimklAuthenticationCode, timeout: number): Promise<SimklClientAuthentication | null>;
    /**
     * Initiates polling with the code obtained by {@link getDeviceCode} to complete device authentication.
     *
     * @param  poll - The device authentication information.
     *
     * @returns  A promise resolving to the completed authentication information or `undefined`.
     */
    pollWithDeviceCode(poll?: SimklAuthenticationCode): CancellablePolling;
    /**
     * Imports the provided authentication information into the client.
     * If the access token is expired, it attempts to refresh it.
     *
     * @param auth - The authentication information to import.
     *
     * @returns A promise resolving to the imported authentication information.
     */
    importAuthentication(auth?: SimklClientAuthentication): SimklClientAuthentication;
}

export { type CancellablePolling, SimklClient };
